from typing import List

from pyEthioNews.news import AsyncBaseScraper
from pyEthioNews.types import News


class Mereja(AsyncBaseScraper):
    def __init__(self):
        super().__init__()
        self.url = "https://www.merega.com/"

    async def get_news(
        self,
        limit: int = 10,
        *args,
        **kwargs,
    ) -> List[News]:
        url = (
            "http://mereja.com/amharic/v2/wp-json/wp/v2/posts"
            + "?per_page="
            + str(limit)
        )
        return await self._fetch_news(url)

    async def get_news_by_id(self, id_: int) -> News:
        url = self.url + "wp-json/wp/v2/posts/{}".format(id_)
        return await self._fetch_news(url)



from datetime import datetime
import json
from dataclasses import dataclass

from bs4 import BeautifulSoup

from typing import Optional, List
from pyEthioNews.news import AsyncBaseScraper


@dataclass
class VoaNews:
    link: str
    title: str
    image: str
    date: str
    _client: "VOAAmharic" = None

    async def fetch(self) -> "VoaNewsDetail":
        if not self._client:
            raise ValueError("You must set a client")
        return await self._client.fetch_news(self)


@dataclass
class VoaNewsDetail:
    title: str
    image: str
    date: datetime
    html: str
    modified: Optional[datetime] = None

    @property
    def content(self):
        return BeautifulSoup(self.html, "html.parser").text


class VOAAmharic(AsyncBaseScraper):
    def __init__(self):
        super().__init__()
        self.url = "https://amharic.voanews.com/{}"

    async def get_world_wide_news(self, page: Optional[int] = 0) -> List[VoaNews]:
        res = await self._s.get(self.url.format("z/3737"), params={"p": page})
        return self._fetch_news_form_html(res.text)

    def _fetch_news_form_html(self, html: str) -> List[VoaNews]:
        soup = self._soup(html)
        div = soup.find_all("div", class_="media-block")
        return [
            VoaNews(
                link=i.find("a")["href"],
                title=i.find("h4").get_text(strip=True),
                image=i.find("img", {"data-src": True})["data-src"],
                date=i.find("span", class_="date").text,
                _client=self,
            )
            for i in div
        ]

    def _fetch_details_from_html(self, html: str) -> VoaNewsDetail:
        soup = self._soup(html)
        title = soup.find("title").text
        content = soup.find("div", class_="wsw").prettify()
        json_data = json.loads(
            soup.find("script", {"type": "application/ld+json"}).string
        )
        date_published = json_data["datePublished"]
        date_modified = json_data["dateModified"]
        image = json_data["image"]["url"]
        return VoaNewsDetail(
            title=title,
            image=image,
            date=datetime.strptime(date_published, "%Y-%m-%d"),
            html=content,
            modified=datetime.strptime(date_modified, "%Y-%m-%d %H:%M:%SZ"),
        )

    async def fetch_news(self, news: VoaNews) -> VoaNewsDetail:
        res = await self._s.get(self.url.format(news.link))
        return self._fetch_details_from_html(res.text)

    async def get(self, slug: str) -> VoaNewsDetail:
        res = await self._s.get(self.url.format(slug))
        return self._fetch_details_from_html(res.text)

    async def search(self, query: str, page: int = 1, limit: int = 10) -> List[VoaNews]:
        res = await self._s.get(
            self.url.format("s"),
            params={"k": query, "tab": "all", "pi": page, "r": "any", "pp": limit},
        )
        return self._fetch_news_form_html(res.text)



from typing import Optional, List

from pyEthioNews.types import NewsType, News
from pyEthioNews.news import AsyncBaseScraper


class FanaBc(AsyncBaseScraper):
    def __init__(self):
        super().__init__()
        self.url = "https://www.fanabc.com/"
        self.categorises = {
            "business": 55,
            "sport": 56,
        }

    async def get_main_news(
        self,
        limit: int = 10,
    ) -> List[News]:
        url = self.url + "wp-json/wp/v2/posts" + "?per_page=" + str(limit)
        return await self._fetch_news(url)

    async def get_business_news(
        self,
        limit: int = 10,
    ) -> List[News]:
        url = self.get_url(self.categorises["business"], limit)
        return await self._fetch_news(url)

    async def get_sport_news(
        self,
        limit: int = 10,
    ) -> List[News]:
        url = self.get_url(self.categorises["sport"], limit)
        return await self._fetch_news(url)

    async def get_news_by_id(self, id_: int) -> News:
        url = self.url + "wp-json/wp/v2/posts/{}".format(id_)
        return await self._fetch_news(url)

    async def get_news(
        self,
        limit: int = 10,
        news: Optional[News] = NewsType.LOCAL,
    ) -> List[News]:
        if news == NewsType.LOCAL:
            return await self.get_main_news(limit)

        elif news == NewsType.WORLD_WIDE:
            return await self.get_main_news(limit)

        elif news == NewsType.BUSINESS:
            return await self.get_business_news(limit)

        elif news == NewsType.SPORT:
            return await self.get_sport_news(limit)



import json
from typing import Union, List, Optional
import httpx
from bs4 import BeautifulSoup
from .types import News


class AsyncBaseScraper:
    def __init__(self):
        self._url = None
        self._headers = {"User-Agent": "Mozilla/5.0"}
        self._s = httpx.AsyncClient(follow_redirects=True)

    @staticmethod
    def _soup(html: str) -> BeautifulSoup:
        return BeautifulSoup(html, "html.parser")

    async def _fetch_news(self, url: str) -> Union[List[News], News]:
        r = await self._s.get(url, headers=self._headers)
        results_json = json.loads(r.text)
        if isinstance(results_json, dict):
            news = News(results_json)
            return news
        news = []
        for result in results_json:
            news.append(News(result))
        return news

    def get_url(self, news_id: int, limit: Optional[Union[int, str]] = None) -> str:
        if limit:
            return (
                f"{self._url}wp-json/wp/v2/posts?categories={news_id}&per_page={limit}"
            )
        return f"{self._url}wp-json/wp/v2/posts?categories={news_id}"



from typing import Union, Optional, List
from pyEthioNews.news import AsyncBaseScraper
from pyEthioNews.types import WaltaNewsType, WaltaSportNewsType, News, NewsType


class Walta(AsyncBaseScraper):
    def __init__(self):
        super().__init__()
        self.url = "https://waltainfo.com/"
        self.categorises = {
            "local": {
                WaltaNewsType.NEWS.value: 158,
                WaltaNewsType.ECONOMY.value: 162,
                WaltaNewsType.POLITICS.value: 160,
                WaltaNewsType.SOCIAL.value: 158,
            },
            "africa": {
                WaltaNewsType.NEWS.value: 167,
                WaltaNewsType.ECONOMY.value: 169,
                WaltaNewsType.POLITICS.value: 171,
                WaltaNewsType.SOCIAL.value: 173,
            },
            "world": {
                WaltaNewsType.NEWS.value: 177,
                WaltaNewsType.ECONOMY.value: 179,
                WaltaNewsType.POLITICS.value: 181,
                WaltaNewsType.SOCIAL.value: 183,
            },
            "sport": {
                WaltaSportNewsType.FOOTBALL.value: 199,
                WaltaSportNewsType.LOCAL.value: 352,
                WaltaSportNewsType.INTERNATIONAL.value: 350,
            },
        }

    async def get_local_news(
        self, limit: int = 10, type_: Optional[WaltaNewsType] = WaltaNewsType.NEWS
    ) -> List[News]:
        url = self.get_url(self.categorises["local"][type_.value], limit)
        return await self._fetch_news(url)

    async def get_africa_news(
        self, limit: int = 10, type_: Optional[WaltaNewsType] = WaltaNewsType.NEWS
    ) -> List[News]:
        url = self.get_url(self.categorises["africa"][type_.value], limit)
        return await self._fetch_news(url)

    async def get_world_wide_news(
        self, limit: int = 10, type_: Optional[WaltaNewsType] = WaltaNewsType.NEWS
    ) -> List[News]:
        url = self.get_url(self.categorises["world"][type_.value], limit)
        return await self._fetch_news(url)

    async def get_sport_news(
        self,
        limit: int = 10,
        type_: Optional[WaltaSportNewsType] = WaltaSportNewsType.FOOTBALL,
    ) -> List[News]:
        url = self.get_url(self.categorises["sport"][type_.value], limit)
        return await self._fetch_news(url)

    async def get_business_news(self, limit: int = 10) -> List[News]:
        url = (
            self.url + "wp-json/wp/v2/posts?categories=303" + "&per_page=" + str(limit)
        )
        return await self._fetch_news(url)

    async def get_news_by_id(self, id_: int) -> News:
        url = self.url + "wp-json/wp/v2/posts/{}".format(id_)
        return await self._fetch_news(url)

    async def get_news(
        self,
        type_: Optional[Union[WaltaNewsType, WaltaSportNewsType]] = None,
        limit: int = 10,
        news: Optional[News] = NewsType.LOCAL,
    ) -> List[News]:
        if type_ is None:
            type_ = WaltaNewsType.NEWS

        if news == NewsType.LOCAL:
            return await self.get_local_news(limit, type_)

        elif news == NewsType.WORLD_WIDE:
            return await self.get_world_wide_news(limit, type_)

        elif news == NewsType.BUSINESS:
            return await self.get_business_news(limit)

        elif news == NewsType.SPORT:
            if not type_ or not isinstance(type_, WaltaSportNewsType):
                type_ = WaltaSportNewsType.FOOTBALL
            return await self.get_sport_news(limit, type_)



import html
import json
import re
from datetime import datetime
from enum import Enum
from typing import Dict, Any

import httpx
from bs4 import BeautifulSoup


class WaltaNewsType(Enum):
    NEWS = "News"
    ECONOMY = "Economy"
    POLITICS = "Politics"
    SOCIAL = "Social"


class WaltaSportNewsType(Enum):
    FOOTBALL = "Football"
    LOCAL = "Social"
    INTERNATIONAL = "International"


class Author:
    def __init__(self, data: Dict[str, Any]):
        self.name = data["name"]
        self.slug = data["slug"]
        self.id_ = data.get("ID") or data.get("id")


class NewsType(Enum):
    LOCAL = "Local"
    WORLD_WIDE = "World"
    BUSINESS = "Business"
    SPORT = "Sport"


class Categories:
    def __init__(self, data: Dict[str, Any]):
        self.name = data["name"]
        self.slug = data["slug"]
        self.id_ = data.get("ID") or data.get("id")


class News:
    def __init__(self, data: Dict[str, Any]):
        self.data = data
        self.id_ = data["id"]
        self.date = data["date"]
        self.slug = data["slug"]
        self.modified = data["modified"]
        self.domain = re.findall(r"^(?:https|http):\/\/(.*.com)\/", data["link"])[0]
        self.url = data["link"]
        self.title = data["title"]["rendered"].replace("&#8211;", "-")
        self.content = data["content"]["rendered"]
        self.excerpt = data["excerpt"]["rendered"].replace("&#8211;", "-")
        self.author = Author(data.get("pwapp_author"))
        self.pwapp_terms = data.get("pwapp_terms")
        self.categories = (
            data["pwapp_terms"]["category"]
            if self.pwapp_terms
            else self.data["categories"]
        )
        self.tags = data["tags"]
        self.comment_status = data["comment_status"]
        self.format = data["format"]
        self.status = data["status"]
        self.image = (
            data.get("share_image")
            if not data.get("featured_image")
            else data["featured_image"]
        )
        self.clean_title()

    def clean_content(self):
        soup = BeautifulSoup(self.content, "lxml")
        return html.unescape(soup.text).replace("&#8211;", "-")

    def get_image(self):
        if not self.image:
            soup = BeautifulSoup(self.content)
            return soup.find("img")["src"]
        return self.image["source"]

    async def get_author(self):
        if not self.author:
            author = self.data["author"]
            url = (
                "https://"
                + self.domain
                + "/amharic/v2/wp-json/wp/v2/users/"
                + str(author)
            )
            s = httpx.AsyncClient()
            res = await s.get(url)
            results_json = json.loads(res.text)
            await s.aclose()
            return Author(results_json)
        return Author(self.author)

    def clean_excerpt(self):
        soup = BeautifulSoup(self.excerpt, "lxml")
        return html.unescape(soup.text).replace("&#8211;", "-")

    def clean_title(self):
        return html.unescape(self.title).replace("&#8211;", "-")

    def convert_date(self):
        return datetime.strptime(self.date, "%Y-%m-%dT%H:%M:%S").timestamp()

    async def get_category(self):
        if not self.categories:
            url = (
                "https://"
                + self.domain
                + "/amharic/v2/wp-json/wp/v2/categories?post="
                + str(self.id_)
            )
            s = httpx.AsyncClient()
            res = await s.get(url)
            results_json = json.loads(res.text)
            await s.aclose()
            return [Categories(cat) for cat in results_json]
        return [Categories(cat) for cat in self.categories]

    def convert_modified_date(self):
        return datetime.strptime(self.modified, "%Y-%m-%dT%H:%M:%S").timestamp()



from pathlib import Path

from setuptools import setup, find_packages

setup(
    name="pyEthioNews",
    version="1.0.1",
    url="https://github.com/wizkiye/pyEthioNewsApi",
    license="MIT",
    author="https://github.com/wizkiye",
    author_email="wizkiye@gmail.com",
    description="Ethiopian News Scraper",
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    install_requires=[
        "httpx",
        "bs4",
        "lxml",
    ],
    packages=find_packages(),
)
